@@ -8,11 +8,11 @@
  *	  This file contains only the public interface routines.
  *
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/access/nbtree/nbtree.c,v 1.87 2002/01/06 00:37:43 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/access/nbtree/nbtree.c,v 1.93 2002/10/20 20:47:31 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -271,6 +271,7 @@ btinsert(PG_FUNCTION_ARGS)
 	char	   *nulls = (char *) PG_GETARG_POINTER(2);
 	ItemPointer ht_ctid = (ItemPointer) PG_GETARG_POINTER(3);
 	Relation	heapRel = (Relation) PG_GETARG_POINTER(4);
+	bool		checkUnique = PG_GETARG_BOOL(5);
 	InsertIndexResult res;
 	BTItem		btitem;
 	IndexTuple	itup;
@@ -280,7 +281,7 @@ btinsert(PG_FUNCTION_ARGS)
 	itup->t_tid = *ht_ctid;
 	btitem = _bt_formitem(itup);
 
-	res = _bt_doinsert(rel, btitem, rel->rd_uniqueindex, heapRel);
+	res = _bt_doinsert(rel, btitem, checkUnique, heapRel);
 
 	pfree(btitem);
 	pfree(itup);
@@ -296,14 +297,16 @@ btgettuple(PG_FUNCTION_ARGS)
 {
 	IndexScanDesc scan = (IndexScanDesc) PG_GETARG_POINTER(0);
 	ScanDirection dir = (ScanDirection) PG_GETARG_INT32(1);
-	RetrieveIndexResult res;
+	BTScanOpaque so = (BTScanOpaque) scan->opaque;
+	Page		page;
+	OffsetNumber offnum;
+	bool		res;
 
 	/*
 	 * If we've already initialized this scan, we can just advance it in
 	 * the appropriate direction.  If we haven't done so yet, we call a
 	 * routine to get the first item in the scan.
 	 */
-
 	if (ItemPointerIsValid(&(scan->currentItemData)))
 	{
 		/*
@@ -312,11 +315,51 @@ btgettuple(PG_FUNCTION_ARGS)
 		 * buffer, too.
 		 */
 		_bt_restscan(scan);
+
+		/*
+		 * Check to see if we should kill the previously-fetched tuple.
+		 */
+		if (scan->kill_prior_tuple)
+		{
+			/*
+			 * Yes, so mark it by setting the LP_DELETE bit in the item
+			 * flags.
+			 */
+			offnum = ItemPointerGetOffsetNumber(&(scan->currentItemData));
+			page = BufferGetPage(so->btso_curbuf);
+			PageGetItemId(page, offnum)->lp_flags |= LP_DELETE;
+
+			/*
+			 * Since this can be redone later if needed, it's treated the
+			 * same as a commit-hint-bit status update for heap tuples: we
+			 * mark the buffer dirty but don't make a WAL log entry.
+			 */
+			SetBufferCommitInfoNeedsSave(so->btso_curbuf);
+		}
+
+		/*
+		 * Now continue the scan.
+		 */
 		res = _bt_next(scan, dir);
 	}
 	else
 		res = _bt_first(scan, dir);
 
+	/*
+	 * Skip killed tuples if asked to.
+	 */
+	if (scan->ignore_killed_tuples)
+	{
+		while (res)
+		{
+			offnum = ItemPointerGetOffsetNumber(&(scan->currentItemData));
+			page = BufferGetPage(so->btso_curbuf);
+			if (!ItemIdDeleted(PageGetItemId(page, offnum)))
+				break;
+			res = _bt_next(scan, dir);
+		}
+	}
+
 	/*
 	 * Save heap TID to use it in _bt_restscan.  Then release the read
 	 * lock on the buffer so that we aren't blocking other backends.
@@ -326,12 +369,12 @@ btgettuple(PG_FUNCTION_ARGS)
 	 */
 	if (res)
 	{
-		((BTScanOpaque) scan->opaque)->curHeapIptr = res->heap_iptr;
+		((BTScanOpaque) scan->opaque)->curHeapIptr = scan->xs_ctup.t_self;
 		LockBuffer(((BTScanOpaque) scan->opaque)->btso_curbuf,
 				   BUFFER_LOCK_UNLOCK);
 	}
 
-	PG_RETURN_POINTER(res);
+	PG_RETURN_BOOL(res);
 }
 
 /*
@@ -341,13 +384,12 @@ Datum
 btbeginscan(PG_FUNCTION_ARGS)
 {
 	Relation	rel = (Relation) PG_GETARG_POINTER(0);
-	bool		fromEnd = PG_GETARG_BOOL(1);
-	uint16		keysz = PG_GETARG_UINT16(2);
-	ScanKey		scankey = (ScanKey) PG_GETARG_POINTER(3);
+	int			keysz = PG_GETARG_INT32(1);
+	ScanKey		scankey = (ScanKey) PG_GETARG_POINTER(2);
 	IndexScanDesc scan;
 
 	/* get the scan */
-	scan = RelationGetIndexScan(rel, fromEnd, keysz, scankey);
+	scan = RelationGetIndexScan(rel, keysz, scankey);
 
 	PG_RETURN_POINTER(scan);
 }
@@ -359,11 +401,7 @@ Datum
 btrescan(PG_FUNCTION_ARGS)
 {
 	IndexScanDesc scan = (IndexScanDesc) PG_GETARG_POINTER(0);
-
-#ifdef NOT_USED					/* XXX surely it's wrong to ignore this? */
-	bool		fromEnd = PG_GETARG_BOOL(1);
-#endif
-	ScanKey		scankey = (ScanKey) PG_GETARG_POINTER(2);
+	ScanKey		scankey = (ScanKey) PG_GETARG_POINTER(1);
 	ItemPointer iptr;
 	BTScanOpaque so;
 
@@ -373,11 +411,13 @@ btrescan(PG_FUNCTION_ARGS)
 	{
 		so = (BTScanOpaque) palloc(sizeof(BTScanOpaqueData));
 		so->btso_curbuf = so->btso_mrkbuf = InvalidBuffer;
-		so->keyData = (ScanKey) NULL;
+		ItemPointerSetInvalid(&(so->curHeapIptr));
+		ItemPointerSetInvalid(&(so->mrkHeapIptr));
 		if (scan->numberOfKeys > 0)
 			so->keyData = (ScanKey) palloc(scan->numberOfKeys * sizeof(ScanKeyData));
+		else
+			so->keyData = (ScanKey) NULL;
 		scan->opaque = so;
-		scan->flags = 0x0;
 	}
 
 	/* we aren't holding any read locks, but gotta drop the pins */
@@ -385,6 +425,7 @@ btrescan(PG_FUNCTION_ARGS)
 	{
 		ReleaseBuffer(so->btso_curbuf);
 		so->btso_curbuf = InvalidBuffer;
+		ItemPointerSetInvalid(&(so->curHeapIptr));
 		ItemPointerSetInvalid(iptr);
 	}
 
@@ -392,6 +433,7 @@ btrescan(PG_FUNCTION_ARGS)
 	{
 		ReleaseBuffer(so->btso_mrkbuf);
 		so->btso_mrkbuf = InvalidBuffer;
+		ItemPointerSetInvalid(&(so->mrkHeapIptr));
 		ItemPointerSetInvalid(iptr);
 	}
 
@@ -491,7 +533,7 @@ btmarkpos(PG_FUNCTION_ARGS)
 	/* bump pin on current buffer for assignment to mark buffer */
 	if (ItemPointerIsValid(&(scan->currentItemData)))
 	{
-		so->btso_mrkbuf = ReadBuffer(scan->relation,
+		so->btso_mrkbuf = ReadBuffer(scan->indexRelation,
 								  BufferGetBlockNumber(so->btso_curbuf));
 		scan->currentMarkData = scan->currentItemData;
 		so->mrkHeapIptr = so->curHeapIptr;
@@ -523,7 +565,7 @@ btrestrpos(PG_FUNCTION_ARGS)
 	/* bump pin on marked buffer */
 	if (ItemPointerIsValid(&(scan->currentMarkData)))
 	{
-		so->btso_curbuf = ReadBuffer(scan->relation,
+		so->btso_curbuf = ReadBuffer(scan->indexRelation,
 								  BufferGetBlockNumber(so->btso_mrkbuf));
 		scan->currentItemData = scan->currentMarkData;
 		so->curHeapIptr = so->mrkHeapIptr;
@@ -549,7 +591,6 @@ btbulkdelete(PG_FUNCTION_ARGS)
 	BlockNumber num_pages;
 	double		tuples_removed;
 	double		num_index_tuples;
-	RetrieveIndexResult res;
 	IndexScanDesc scan;
 	BTScanOpaque so;
 	ItemPointer current;
@@ -562,27 +603,33 @@ btbulkdelete(PG_FUNCTION_ARGS)
 	 * loop, we skip most of the wrapper layers of index_getnext and
 	 * instead call _bt_step directly.	This implies holding buffer lock
 	 * on a target page throughout the loop over the page's tuples.
-	 * Initially, we have a read lock acquired by _bt_step when we stepped
-	 * onto the page.  If we find a tuple we need to delete, we trade in
-	 * the read lock for an exclusive write lock; after that, we hold the
-	 * write lock until we step off the page (fortunately, _bt_relbuf
-	 * doesn't care which kind of lock it's releasing).  This should
-	 * minimize the amount of work needed per page.
+	 *
+	 * Whenever we step onto a new page, we have to trade in the read
+	 * lock acquired by _bt_first or _bt_step for an exclusive write lock
+	 * (fortunately, _bt_relbuf doesn't care which kind of lock it's
+	 * releasing when it comes time for _bt_step to release our lock).
+	 *
+	 * Note that we exclusive-lock every leaf page, or at least every one
+	 * containing data items.  It sounds attractive to only exclusive-lock
+	 * those containing items we need to delete, but unfortunately that
+	 * is not safe: we could then pass a stopped indexscan, which could
+	 * in rare cases lead to deleting the item it needs to find when it
+	 * resumes.  (See _bt_restscan --- this could only happen if an indexscan
+	 * stops on a deletable item and then a page split moves that item
+	 * into a page further to its right, which the indexscan will have no
+	 * pin on.)
 	 */
-	scan = index_beginscan(rel, false, 0, (ScanKey) NULL);
+	scan = index_beginscan(NULL, rel, SnapshotAny, 0, (ScanKey) NULL);
 	so = (BTScanOpaque) scan->opaque;
 	current = &(scan->currentItemData);
 
 	/* Use _bt_first to get started, then _bt_step to remaining tuples */
-	res = _bt_first(scan, ForwardScanDirection);
-
-	if (res != NULL)
+	if (_bt_first(scan, ForwardScanDirection))
 	{
 		Buffer		buf;
 		BlockNumber lockedBlock = InvalidBlockNumber;
 
-		pfree(res);
-		/* we have the buffer pinned and locked */
+		/* we have the buffer pinned and read-locked */
 		buf = so->btso_curbuf;
 		Assert(BufferIsValid(buf));
 
@@ -599,65 +646,59 @@ btbulkdelete(PG_FUNCTION_ARGS)
 			CHECK_FOR_INTERRUPTS();
 
 			/* current is the next index tuple */
-			blkno = ItemPointerGetBlockNumber(current);
-			offnum = ItemPointerGetOffsetNumber(current);
 			page = BufferGetPage(buf);
+			blkno = ItemPointerGetBlockNumber(current);
+
+			/*
+			 * Make sure we have a super-exclusive write lock on this page.
+			 *
+			 * We assume that only concurrent insertions, not deletions,
+			 * can occur while we're not holding the page lock (the
+			 * caller should hold a suitable relation lock to ensure
+			 * this). Therefore, no items can escape being scanned because
+			 * of this temporary lock release.
+			 */
+			if (blkno != lockedBlock)
+			{
+				LockBuffer(buf, BUFFER_LOCK_UNLOCK);
+				LockBufferForCleanup(buf);
+				lockedBlock = blkno;
+				/*
+				 * If the page was formerly rightmost but was split while we
+				 * didn't hold the lock, and ip_posid is pointing to item
+				 * 1, then ip_posid now points at the high key not a valid
+				 * data item. In this case we need to step forward.
+				 */
+				opaque = (BTPageOpaque) PageGetSpecialPointer(page);
+				if (current->ip_posid < P_FIRSTDATAKEY(opaque))
+					current->ip_posid = P_FIRSTDATAKEY(opaque);
+			}
+
+			offnum = ItemPointerGetOffsetNumber(current);
 			btitem = (BTItem) PageGetItem(page, PageGetItemId(page, offnum));
 			itup = &btitem->bti_itup;
 			htup = &(itup->t_tid);
 
 			if (callback(htup, callback_state))
 			{
-				/*
-				 * If this is first deletion on this page, trade in read
-				 * lock for a really-exclusive write lock.	Then, step
-				 * back one and re-examine the item, because other backends
-				 * might have inserted item(s) while we weren't holding
-				 * the lock!
-				 *
-				 * We assume that only concurrent insertions, not deletions,
-				 * can occur while we're not holding the page lock (the caller
-				 * should hold a suitable relation lock to ensure this).
-				 * Therefore, the item we want to delete is either in the
-				 * same slot as before, or some slot to its right.
-				 * Rechecking the same slot is necessary and sufficient to
-				 * get back in sync after any insertions.
-				 */
-				if (blkno != lockedBlock)
-				{
-					LockBuffer(buf, BUFFER_LOCK_UNLOCK);
-					LockBufferForCleanup(buf);
-					lockedBlock = blkno;
-				}
-				else
-				{
-					/* Okay to delete the item from the page */
-					_bt_itemdel(rel, buf, current);
+				/* Okay to delete the item from the page */
+				_bt_itemdel(rel, buf, current);
 
-					/* Mark buffer dirty, but keep the lock and pin */
-					WriteNoReleaseBuffer(buf);
+				/* Mark buffer dirty, but keep the lock and pin */
+				WriteNoReleaseBuffer(buf);
 
-					tuples_removed += 1;
-				}
+				tuples_removed += 1;
 
 				/*
-				 * In either case, we now need to back up the scan one item,
-				 * so that the next cycle will re-examine the same offnum on
-				 * this page.
+				 * We now need to back up the scan one item, so that the next
+				 * cycle will re-examine the same offnum on this page (which
+				 * now holds the next item).
 				 *
 				 * For now, just hack the current-item index.  Will need to
 				 * be smarter when deletion includes removal of empty
 				 * index pages.
-				 *
-				 * We must decrement ip_posid in all cases but one: if the
-				 * page was formerly rightmost but was split while we didn't
-				 * hold the lock, and ip_posid is pointing to item 1, then
-				 * ip_posid now points at the high key not a valid data item.
-				 * In this case we do want to step forward.
 				 */
-				opaque = (BTPageOpaque) PageGetSpecialPointer(page);
-				if (current->ip_posid >= P_FIRSTDATAKEY(opaque))
-					current->ip_posid--;
+				current->ip_posid--;
 			}
 			else
 				num_index_tuples += 1;
@@ -679,11 +720,21 @@ btbulkdelete(PG_FUNCTION_ARGS)
 
 /*
  * Restore scan position when btgettuple is called to continue a scan.
+ *
+ * This is nontrivial because concurrent insertions might have moved the
+ * index tuple we stopped on.  We assume the tuple can only have moved to
+ * the right from our stop point, because we kept a pin on the buffer,
+ * and so no deletion can have occurred on that page.
+ *
+ * On entry, we have a pin but no read lock on the buffer that contained
+ * the index tuple we stopped the scan on.  On exit, we have pin and read
+ * lock on the buffer that now contains that index tuple, and the scandesc's
+ * current position is updated to point at it.
  */
 static void
 _bt_restscan(IndexScanDesc scan)
 {
-	Relation	rel = scan->relation;
+	Relation	rel = scan->indexRelation;
 	BTScanOpaque so = (BTScanOpaque) scan->opaque;
 	Buffer		buf = so->btso_curbuf;
 	Page		page;
@@ -691,13 +742,14 @@ _bt_restscan(IndexScanDesc scan)
 	OffsetNumber offnum = ItemPointerGetOffsetNumber(current),
 				maxoff;
 	BTPageOpaque opaque;
+	Buffer		nextbuf;
 	ItemPointerData target = so->curHeapIptr;
 	BTItem		item;
 	BlockNumber blkno;
 
 	/*
-	 * Get back the read lock we were holding on the buffer. (We still
-	 * have a reference-count pin on it, so need not get that.)
+	 * Reacquire read lock on the buffer.  (We should still have
+	 * a reference-count pin on it, so need not get that.)
 	 */
 	LockBuffer(buf, BT_READ);
 
@@ -709,7 +761,7 @@ _bt_restscan(IndexScanDesc scan)
 	 * We use this as flag when first index tuple on page is deleted but
 	 * we do not move left (this would slowdown vacuum) - so we set
 	 * current->ip_posid before first index tuple on the current page
-	 * (_bt_step will move it right)...
+	 * (_bt_step will move it right)...  XXX still needed?
 	 */
 	if (!ItemPointerIsValid(&target))
 	{
@@ -720,7 +772,7 @@ _bt_restscan(IndexScanDesc scan)
 
 	/*
 	 * The item we were on may have moved right due to insertions. Find it
-	 * again.
+	 * again.  We use the heap TID to identify the item uniquely.
 	 */
 	for (;;)
 	{
@@ -736,28 +788,33 @@ _bt_restscan(IndexScanDesc scan)
 				target.ip_blkid.bi_lo &&
 				item->bti_itup.t_tid.ip_posid == target.ip_posid)
 			{
+				/* Found it */
 				current->ip_posid = offnum;
 				return;
 			}
 		}
 
 		/*
-		 * By here, the item we're looking for moved right at least one
-		 * page
+		 * The item we're looking for moved right at least one page, so
+		 * move right.  We are careful here to pin and read-lock the next
+		 * page before releasing the current one.  This ensures that a
+		 * concurrent btbulkdelete scan cannot pass our position --- if it
+		 * did, it might be able to reach and delete our target item before
+		 * we can find it again.
 		 */
 		if (P_RIGHTMOST(opaque))
 			elog(FATAL, "_bt_restscan: my bits moved right off the end of the world!"
 				 "\n\tRecreate index %s.", RelationGetRelationName(rel));
 
 		blkno = opaque->btpo_next;
+		nextbuf = _bt_getbuf(rel, blkno, BT_READ);
 		_bt_relbuf(rel, buf);
-		buf = _bt_getbuf(rel, blkno, BT_READ);
+		so->btso_curbuf = buf = nextbuf;
 		page = BufferGetPage(buf);
 		maxoff = PageGetMaxOffsetNumber(page);
 		opaque = (BTPageOpaque) PageGetSpecialPointer(page);
 		offnum = P_FIRSTDATAKEY(opaque);
 		ItemPointerSet(current, blkno, offnum);
-		so->btso_curbuf = buf;
 	}
 }
 
@@ -776,7 +833,7 @@ _bt_restore_page(Page page, char *from, int len)
 		itemsz = MAXALIGN(itemsz);
 		if (PageAddItem(page, (Item) from, itemsz,
 					  FirstOffsetNumber, LP_USED) == InvalidOffsetNumber)
-			elog(STOP, "_bt_restore_page: can't add item to page");
+			elog(PANIC, "_bt_restore_page: can't add item to page");
 		from += itemsz;
 	}
 }
@@ -799,10 +856,10 @@ btree_xlog_delete(bool redo, XLogRecPtr lsn, XLogRecord *record)
 	buffer = XLogReadBuffer(false, reln,
 						ItemPointerGetBlockNumber(&(xlrec->target.tid)));
 	if (!BufferIsValid(buffer))
-		elog(STOP, "btree_delete_redo: block unfound");
+		elog(PANIC, "btree_delete_redo: block unfound");
 	page = (Page) BufferGetPage(buffer);
 	if (PageIsNew((PageHeader) page))
-		elog(STOP, "btree_delete_redo: uninitialized page");
+		elog(PANIC, "btree_delete_redo: uninitialized page");
 
 	if (XLByteLE(lsn, PageGetLSN(page)))
 	{
@@ -838,10 +895,10 @@ btree_xlog_insert(bool redo, XLogRecPtr lsn, XLogRecord *record)
 	buffer = XLogReadBuffer(false, reln,
 						ItemPointerGetBlockNumber(&(xlrec->target.tid)));
 	if (!BufferIsValid(buffer))
-		elog(STOP, "btree_insert_%sdo: block unfound", (redo) ? "re" : "un");
+		elog(PANIC, "btree_insert_%sdo: block unfound", (redo) ? "re" : "un");
 	page = (Page) BufferGetPage(buffer);
 	if (PageIsNew((PageHeader) page))
-		elog(STOP, "btree_insert_%sdo: uninitialized page", (redo) ? "re" : "un");
+		elog(PANIC, "btree_insert_%sdo: uninitialized page", (redo) ? "re" : "un");
 	pageop = (BTPageOpaque) PageGetSpecialPointer(page);
 
 	if (redo)
@@ -855,7 +912,7 @@ btree_xlog_insert(bool redo, XLogRecPtr lsn, XLogRecord *record)
 						record->xl_len - SizeOfBtreeInsert,
 						ItemPointerGetOffsetNumber(&(xlrec->target.tid)),
 						LP_USED) == InvalidOffsetNumber)
-			elog(STOP, "btree_insert_redo: failed to add item");
+			elog(PANIC, "btree_insert_redo: failed to add item");
 
 		PageSetLSN(page, lsn);
 		PageSetSUI(page, ThisStartUpID);
@@ -864,7 +921,7 @@ btree_xlog_insert(bool redo, XLogRecPtr lsn, XLogRecord *record)
 	else
 	{
 		if (XLByteLT(PageGetLSN(page), lsn))
-			elog(STOP, "btree_insert_undo: bad page LSN");
+			elog(PANIC, "btree_insert_undo: bad page LSN");
 
 		if (!P_ISLEAF(pageop))
 		{
@@ -872,7 +929,7 @@ btree_xlog_insert(bool redo, XLogRecPtr lsn, XLogRecord *record)
 			return;
 		}
 
-		elog(STOP, "btree_insert_undo: unimplemented");
+		elog(PANIC, "btree_insert_undo: unimplemented");
 	}
 
 	return;
@@ -899,13 +956,13 @@ btree_xlog_split(bool redo, bool onleft, XLogRecPtr lsn, XLogRecord *record)
 		BlockIdGetBlockNumber(&(xlrec->otherblk));
 	buffer = XLogReadBuffer(false, reln, blkno);
 	if (!BufferIsValid(buffer))
-		elog(STOP, "btree_split_%s: lost left sibling", op);
+		elog(PANIC, "btree_split_%s: lost left sibling", op);
 
 	page = (Page) BufferGetPage(buffer);
 	if (redo)
 		_bt_pageinit(page, BufferGetPageSize(buffer));
 	else if (PageIsNew((PageHeader) page))
-		elog(STOP, "btree_split_undo: uninitialized left sibling");
+		elog(PANIC, "btree_split_undo: uninitialized left sibling");
 	pageop = (BTPageOpaque) PageGetSpecialPointer(page);
 
 	if (redo)
@@ -928,8 +985,8 @@ btree_xlog_split(bool redo, bool onleft, XLogRecPtr lsn, XLogRecord *record)
 /* undo */
 	{
 		if (XLByteLT(PageGetLSN(page), lsn))
-			elog(STOP, "btree_split_undo: bad left sibling LSN");
-		elog(STOP, "btree_split_undo: unimplemented");
+			elog(PANIC, "btree_split_undo: bad left sibling LSN");
+		elog(PANIC, "btree_split_undo: unimplemented");
 	}
 
 	/* Right (new) sibling */
@@ -937,13 +994,13 @@ btree_xlog_split(bool redo, bool onleft, XLogRecPtr lsn, XLogRecord *record)
 		ItemPointerGetBlockNumber(&(xlrec->target.tid));
 	buffer = XLogReadBuffer((redo) ? true : false, reln, blkno);
 	if (!BufferIsValid(buffer))
-		elog(STOP, "btree_split_%s: lost right sibling", op);
+		elog(PANIC, "btree_split_%s: lost right sibling", op);
 
 	page = (Page) BufferGetPage(buffer);
 	if (redo)
 		_bt_pageinit(page, BufferGetPageSize(buffer));
 	else if (PageIsNew((PageHeader) page))
-		elog(STOP, "btree_split_undo: uninitialized right sibling");
+		elog(PANIC, "btree_split_undo: uninitialized right sibling");
 	pageop = (BTPageOpaque) PageGetSpecialPointer(page);
 
 	if (redo)
@@ -967,8 +1024,8 @@ btree_xlog_split(bool redo, bool onleft, XLogRecPtr lsn, XLogRecord *record)
 /* undo */
 	{
 		if (XLByteLT(PageGetLSN(page), lsn))
-			elog(STOP, "btree_split_undo: bad right sibling LSN");
-		elog(STOP, "btree_split_undo: unimplemented");
+			elog(PANIC, "btree_split_undo: bad right sibling LSN");
+		elog(PANIC, "btree_split_undo: unimplemented");
 	}
 
 	if (!redo || (record->xl_info & XLR_BKP_BLOCK_1))
@@ -981,11 +1038,11 @@ btree_xlog_split(bool redo, bool onleft, XLogRecPtr lsn, XLogRecord *record)
 
 	buffer = XLogReadBuffer(false, reln, blkno);
 	if (!BufferIsValid(buffer))
-		elog(STOP, "btree_split_redo: lost next right page");
+		elog(PANIC, "btree_split_redo: lost next right page");
 
 	page = (Page) BufferGetPage(buffer);
 	if (PageIsNew((PageHeader) page))
-		elog(STOP, "btree_split_redo: uninitialized next right page");
+		elog(PANIC, "btree_split_redo: uninitialized next right page");
 
 	if (XLByteLE(lsn, PageGetLSN(page)))
 	{
@@ -1022,10 +1079,10 @@ btree_xlog_newroot(bool redo, XLogRecPtr lsn, XLogRecord *record)
 		return;
 	buffer = XLogReadBuffer(true, reln, BlockIdGetBlockNumber(&(xlrec->rootblk)));
 	if (!BufferIsValid(buffer))
-		elog(STOP, "btree_newroot_redo: no root page");
+		elog(PANIC, "btree_newroot_redo: no root page");
 	metabuf = XLogReadBuffer(false, reln, BTREE_METAPAGE);
 	if (!BufferIsValid(buffer))
-		elog(STOP, "btree_newroot_redo: no metapage");
+		elog(PANIC, "btree_newroot_redo: no metapage");
 	page = (Page) BufferGetPage(buffer);
 	_bt_pageinit(page, BufferGetPageSize(buffer));
 	pageop = (BTPageOpaque) PageGetSpecialPointer(page);
@@ -1079,7 +1136,7 @@ btree_redo(XLogRecPtr lsn, XLogRecord *record)
 	else if (info == XLOG_BTREE_NEWROOT)
 		btree_xlog_newroot(true, lsn, record);
 	else
-		elog(STOP, "btree_redo: unknown op code %u", info);
+		elog(PANIC, "btree_redo: unknown op code %u", info);
 }
 
 void
@@ -1099,7 +1156,7 @@ btree_undo(XLogRecPtr lsn, XLogRecord *record)
 	else if (info == XLOG_BTREE_NEWROOT)
 		btree_xlog_newroot(false, lsn, record);
 	else
-		elog(STOP, "btree_undo: unknown op code %u", info);
+		elog(PANIC, "btree_undo: unknown op code %u", info);
 }
 
 static void
