@@ -1,7 +1,7 @@
 /* -----------------------------------------------------------------------
  * ascii.c
  *
- * $Header: /cvsroot/pgsql-server/src/backend/utils/adt/ascii.c,v 1.12 2001/11/05 17:46:28 momjian Exp $
+ * $Header: /cvsroot/pgsql-server/src/backend/utils/adt/ascii.c,v 1.13.2.1 2003/04/02 21:08:07 tgl Exp $
  *
  *	 Portions Copyright (c) 1999-2000, PostgreSQL Global Development Group
  *
@@ -18,46 +18,6 @@
 #include "mb/pg_wchar.h"
 #include "utils/ascii.h"
 
-/* ----------
- * even if MULTIBYTE is not enabled, these functions must exist
- * since pg_proc.h has references to them.
- * ----------
- */
-#ifndef MULTIBYTE
-
-static void multibyte_error(void);
-
-static void
-multibyte_error(void)
-{
-	elog(ERROR, "Multi-byte support is not enabled");
-}
-
-Datum
-to_ascii_encname(PG_FUNCTION_ARGS)
-{
-	multibyte_error();
-	return 0;					/* keep compiler quiet */
-}
-
-Datum
-to_ascii_enc(PG_FUNCTION_ARGS)
-{
-	multibyte_error();
-	return 0;					/* keep compiler quiet */
-}
-
-Datum
-to_ascii_default(PG_FUNCTION_ARGS)
-{
-	multibyte_error();
-	return 0;					/* keep compiler quiet */
-}
-
-
-#else							/* with MULTIBYTE */
-
-
 static text *encode_to_ascii(text *data, int enc);
 
 /* ----------
@@ -67,9 +27,9 @@ static text *encode_to_ascii(text *data, int enc);
 char *
 pg_to_ascii(unsigned char *src, unsigned char *src_end, unsigned char *desc, int enc)
 {
-	unsigned char *x = NULL;
-	unsigned char *ascii = NULL;
-	int			range = 0;
+	unsigned char *x;
+	unsigned char *ascii;
+	int			range;
 
 	/*
 	 * relevant start for an encoding
@@ -106,12 +66,13 @@ pg_to_ascii(unsigned char *src, unsigned char *src_end, unsigned char *desc, int
 	{
 		elog(ERROR, "pg_to_ascii(): unsupported encoding from %s",
 			 pg_encoding_to_char(enc));
+		return NULL;			/* keep compiler quiet */
 	}
 
 	/*
 	 * Encode
 	 */
-	for (x = src; x <= src_end; x++)
+	for (x = src; x < src_end; x++)
 	{
 		if (*x < 128)
 			*desc++ = *x;
@@ -190,5 +151,3 @@ to_ascii_default(PG_FUNCTION_ARGS)
 		  )
 		);
 }
-
-#endif   /* MULTIBYTE */
