@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/utils/adt/date.c,v 1.73.2.1 2002/11/21 23:31:37 tgl Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/utils/adt/date.c,v 1.73.2.4 2003/02/13 17:04:24 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -619,7 +619,7 @@ static void
 AdjustTimeForTypmod(TimeADT *time, int32 typmod)
 {
 #ifdef HAVE_INT64_TIMESTAMP
-	static const int64 TimeScales[MAX_TIMESTAMP_PRECISION + 1] = {
+	static const int64 TimeScales[MAX_TIME_PRECISION + 1] = {
 		INT64CONST(1000000),
 		INT64CONST(100000),
 		INT64CONST(10000),
@@ -629,72 +629,58 @@ AdjustTimeForTypmod(TimeADT *time, int32 typmod)
 		INT64CONST(1)
 	};
 
-	static const int64 TimeOffsets[MAX_TIMESTAMP_PRECISION + 1] = {
-		INT64CONST(-500000),
-		INT64CONST(-50000),
-		INT64CONST(-5000),
-		INT64CONST(-500),
-		INT64CONST(-50),
-		INT64CONST(-5),
+	static const int64 TimeOffsets[MAX_TIME_PRECISION + 1] = {
+		INT64CONST(500000),
+		INT64CONST(50000),
+		INT64CONST(5000),
+		INT64CONST(500),
+		INT64CONST(50),
+		INT64CONST(5),
 		INT64CONST(0)
 	};
 
 #else
-	static const double TimeScales[MAX_TIMESTAMP_PRECISION + 1] = {
+	/* note MAX_TIME_PRECISION differs in this case */
+	static const double TimeScales[MAX_TIME_PRECISION + 1] = {
 		1,
 		10,
 		100,
 		1000,
 		10000,
 		100000,
-		1000000
-	};
-
-	static const double TimeOffsets[MAX_TIMESTAMP_PRECISION + 1] = {
-		0.5,
-		0.05,
-		0.005,
-		0.0005,
-		0.00005,
-		0.000005,
-		0.0000005
+		1000000,
+		10000000,
+		100000000,
+		1000000000,
+		10000000000
 	};
 #endif
 
 	if ((typmod >= 0) && (typmod <= MAX_TIME_PRECISION))
 	{
+		/*
+		 * Note: this round-to-nearest code is not completely consistent
+		 * about rounding values that are exactly halfway between integral
+		 * values.  On most platforms, rint() will implement round-to-nearest-even,
+		 * but the integer code always rounds up (away from zero).  Is it
+		 * worth trying to be consistent?
+		 */
 #ifdef HAVE_INT64_TIMESTAMP
-		/* we have different truncation behavior depending on sign */
 		if (*time >= INT64CONST(0))
-		{
-			*time = ((*time / TimeScales[typmod])
-					 * TimeScales[typmod]);
-		}
-		else
 		{
 			*time = (((*time + TimeOffsets[typmod]) / TimeScales[typmod])
 					 * TimeScales[typmod]);
 		}
-#else
-		/* we have different truncation behavior depending on sign */
-		if (*time >= 0)
-		{
-			*time = (rint(((double) *time) * TimeScales[typmod])
-					 / TimeScales[typmod]);
-		}
 		else
 		{
-			/*
-			 * Scale and truncate first, then add to help the rounding
-			 * behavior
-			 */
-			*time = (rint((((double) *time) * TimeScales[typmod]) + TimeOffsets[typmod])
-					 / TimeScales[typmod]);
+			*time = - ((((- *time) + TimeOffsets[typmod]) / TimeScales[typmod])
+					   * TimeScales[typmod]);
 		}
+#else
+		*time = (rint(((double) *time) * TimeScales[typmod])
+				 / TimeScales[typmod]);
 #endif
 	}
-
-	return;
 }
 
 
@@ -1362,23 +1348,22 @@ timetz_out(PG_FUNCTION_ARGS)
 
 /* timetz2tm()
  * Convert TIME WITH TIME ZONE data type to POSIX time structure.
- * For dates within the system-supported time_t range, convert to the
- *	local time zone. If out of this range, leave as GMT. - tgl 97/05/27
  */
 static int
 timetz2tm(TimeTzADT *time, struct tm * tm, fsec_t *fsec, int *tzp)
 {
 #ifdef HAVE_INT64_TIMESTAMP
-	tm->tm_hour = (time->time / INT64CONST(3600000000));
-	time->time -= (tm->tm_hour * INT64CONST(3600000000));
-	tm->tm_min = (time->time / INT64CONST(60000000));
-	time->time -= (tm->tm_min * INT64CONST(60000000));
-	tm->tm_sec = (time->time / INT64CONST(1000000));
-	*fsec = (time->time - (tm->tm_sec * INT64CONST(1000000)));
+	int64		trem = time->time;
+
+	tm->tm_hour = (trem / INT64CONST(3600000000));
+	trem -= (tm->tm_hour * INT64CONST(3600000000));
+	tm->tm_min = (trem / INT64CONST(60000000));
+	trem -= (tm->tm_min * INT64CONST(60000000));
+	tm->tm_sec = (trem / INT64CONST(1000000));
+	*fsec = (trem - (tm->tm_sec * INT64CONST(1000000)));
 #else
-	double		trem;
+	double		trem = time->time;
 
-	trem = time->time;
 	TMODULO(trem, tm->tm_hour, 3600e0);
 	TMODULO(trem, tm->tm_min, 60e0);
 	TMODULO(trem, tm->tm_sec, 1e0);
